"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.fetchFlags = fetchFlags;
exports.flagActivity = flagActivity;
exports.removeFlagFromServer = removeFlagFromServer;
exports.UPDATE_FLAG_STATUS = exports.REMOVE_FLAG = exports.STORE_FLAGS = exports.ADD_FLAG = void 0;
var ADD_FLAG = 'flags/ADD_FLAG';
exports.ADD_FLAG = ADD_FLAG;
var STORE_FLAGS = 'flags/STORE_FLAGS';
exports.STORE_FLAGS = STORE_FLAGS;
var REMOVE_FLAG = 'flags/REMOVE_FLAG';
exports.REMOVE_FLAG = REMOVE_FLAG;
var UPDATE_FLAG_STATUS = 'flags/UPDATE_FLAG_STATUS';
exports.UPDATE_FLAG_STATUS = UPDATE_FLAG_STATUS;

function updateFlagStatus(status) {
  return {
    type: UPDATE_FLAG_STATUS,
    payload: {
      status: status
    }
  };
}

function addFlag(flag, error) {
  return {
    type: ADD_FLAG,
    payload: {
      flag: flag,
      error: error
    }
  };
}

function storeFlags(flags, error) {
  return {
    type: STORE_FLAGS,
    payload: {
      flags: flags,
      error: error
    }
  };
}

function removeFlag(flag, error) {
  return {
    type: REMOVE_FLAG,
    payload: {
      error: error,
      flag: flag
    }
  };
}

function constructFlag(flag) {
  return {
    id: flag.id,
    url: flag.url,
    activityUrl: flag['flag-item'],
    isInFlight: false
  };
}
/**
 * Fetches all of the current user's flags
 *
 * @param {any} sparkInstance
 * @returns {function}
 */


function fetchFlags(sparkInstance) {
  return function (dispatch) {
    dispatch(updateFlagStatus({
      isFetching: true
    }));
    return sparkInstance.internal.flag.list().then(function (flags) {
      var flagsMap = flags.map(constructFlag);
      flagsMap = flagsMap.reduce(function (acc, val) {
        acc[val.activityUrl] = val;
        return acc;
      }, {});
      return dispatch(storeFlags(flagsMap));
    }).catch(function (error) {
      return sparkInstance.logger.warn(error);
    });
  };
}
/**
 * Flags a given activity. Updates state immediately then
 * adds flag details given from api
 *
 * @param {any} activity
 * @param {any} spark
 * @returns {function}
 */


function flagActivity(activity, spark) {
  return function (dispatch) {
    var flagObject = {
      activityUrl: activity.url,
      isInFlight: true
    };
    dispatch(addFlag(flagObject));
    return spark.internal.flag.create(activity).then(function (flag) {
      return dispatch(addFlag(constructFlag(flag)));
    }).catch(function (error) {
      return spark.logger.warn(error);
    });
  };
}
/**
 * Removes a flag from the server. Updates the state immediately
 * but re-adds it if the delete fails
 *
 * @param {any} flag
 * @param {any} spark
 * @returns {function}
 */


function removeFlagFromServer(flag, spark) {
  return function (dispatch) {
    dispatch(removeFlag(flag));
    return spark.internal.flag.delete(flag.toJS()).catch(function (error) {
      spark.logger.warn(error);
    });
  };
}