"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = reducer;
exports.initialState = void 0;

var _immutable = require("immutable");

var _actions = require("./actions");

var initialState = (0, _immutable.fromJS)({
  flags: {},
  error: null,
  status: {
    hasFetched: false,
    isFetching: false
  }
});
exports.initialState = initialState;

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments.length > 1 ? arguments[1] : undefined;

  switch (action.type) {
    case _actions.ADD_FLAG:
      {
        var _action$payload = action.payload,
            flag = _action$payload.flag,
            error = _action$payload.error;

        if (error) {
          return state.setIn(['flags', flag.activityUrl], {
            error: error
          });
        }

        return state.setIn(['flags', flag.activityUrl], (0, _immutable.fromJS)(flag));
      }

    case _actions.STORE_FLAGS:
      {
        var _action$payload2 = action.payload,
            flags = _action$payload2.flags,
            _error = _action$payload2.error;

        if (_error) {
          return state.setIn(['status', 'hasFetched'], true).setIn(['status', 'isFetching'], false).set('error', _error);
        }

        return state.setIn(['status', 'hasFetched'], true).setIn(['status', 'isFetching'], false).mergeDeepIn(['flags'], flags);
      }

    case _actions.REMOVE_FLAG:
      {
        var _flag = action.payload.flag;

        if (action.error) {
          // Unable to delete flag, add it back in
          return state.setIn(['flags', _flag.activityUrl], (0, _immutable.fromJS)(_flag));
        }

        return state.deleteIn(['flags', _flag.get('activityUrl')]);
      }

    case _actions.UPDATE_FLAG_STATUS:
      {
        return state.mergeDeepIn(['status'], action.payload.status);
      }

    default:
      {
        return state;
      }
  }
}