import configureMockStore from 'redux-mock-store';
import {fromJS} from 'immutable';
import thunk from 'redux-thunk';

import {initialState} from './reducer';
import * as actions from './actions';

const middlewares = [thunk];
const mockStore = configureMockStore(middlewares);


let mockSpark, store;

const activity = {
  url: 'https://testurl'
};

const flag = fromJS({
  id: 1,
  url: 'https://testurl',
  activityUrl: 'https://activityUrl1'
});

describe('redux-module-flags actions ', () => {
  beforeEach(() => {
    mockSpark = {
      internal: {
        flag: {
          create: jest.fn(() => Promise.resolve()),
          list: jest.fn(() => Promise.resolve([
            {id: 1, url: 'http://testurl1', activityUrl: 'https://activityUrl1'},
            {id: 2, url: 'http://testurl2', activityUrl: 'https://activityUrl2'},
            {id: 3, url: 'http://testurl3', activityUrl: 'https://activityUrl3'}
          ])),
          delete: jest.fn(() => Promise.resolve())
        }
      },
      logger: {
        warn: jest.fn()
      }
    };
    store = mockStore(initialState);
  });

  it('has exported actions', () => {
    expect(actions.fetchFlags).toBeDefined();
    expect(actions.flagActivity).toBeDefined();
    expect(actions.removeFlagFromServer).toBeDefined();
  });

  describe('#fetchFlags', () => {
    it('can successfully fetch Flags', () =>
      store.dispatch(actions.fetchFlags(mockSpark))
        .then(() => {
          expect(mockSpark.internal.flag.list).toHaveBeenCalled();
          expect(store.getActions()).toMatchSnapshot();
        }));

    it('can error correctly', () => {
      mockSpark.internal.flag.list = jest.fn(() => Promise.reject(new Error()));

      return store.dispatch(actions.fetchFlags(mockSpark))
        .then(() => {
          expect(mockSpark.internal.flag.list).toHaveBeenCalled();
          expect(store.getActions()).toMatchSnapshot();
        });
    });
  });

  describe('#flagActivity', () => {
    it('can successfully flag an activity', () =>
      store.dispatch(actions.flagActivity(activity, mockSpark))
        .then(() => {
          expect(mockSpark.internal.flag.create).toHaveBeenCalled();
          expect(store.getActions()).toMatchSnapshot();
        }));

    it('can error correctly', () => {
      mockSpark.internal.flag.create = jest.fn(() => Promise.reject(new Error()));

      return store.dispatch(actions.flagActivity(activity, mockSpark))
        .then(() => {
          expect(mockSpark.internal.flag.create).toHaveBeenCalled();
          expect(mockSpark.logger.warn).toHaveBeenCalled();
          expect(store.getActions()).toMatchSnapshot();
        });
    });
  });

  describe('#removeFlagFromServer', () => {
    it('can successfully remove flag', () =>
      store.dispatch(actions.removeFlagFromServer(flag, mockSpark))
        .then(() => {
          expect(mockSpark.internal.flag.delete).toHaveBeenCalled();
          expect(store.getActions()).toMatchSnapshot();
        }));

    it('can error correctly', () => {
      mockSpark.internal.flag.delete = jest.fn(() => Promise.reject(new Error()));

      return store.dispatch(actions.removeFlagFromServer(flag, mockSpark))
        .then(() => {
          expect(mockSpark.internal.flag.delete).toHaveBeenCalled();
          expect(mockSpark.logger.warn).toHaveBeenCalled();
          expect(store.getActions()).toMatchSnapshot();
        });
    });
  });
});
